package flowserver

import (
	"encoding/json"
	"flowserver/msg"
	"sync"

	"github.com/gorilla/websocket"
)

// This might be better named as a room/project than a session

// FlowSession Create a session and link clients
type FlowSession struct {
	sync.Mutex
	ID string // Random handle for sessionID
	// List of clients on this session
	clients []*websocket.Conn

	RawData []byte // Just share data
}

//NewSession creates and initializes a NewSession
func NewSession(ID string) *FlowSession {
	s := &FlowSession{sync.Mutex{}, ID, []*websocket.Conn{}, []byte{}}
	return s
}

// ClientAdd add a client to session
func (f *FlowSession) ClientAdd(c *websocket.Conn) error {
	f.Lock()
	defer f.Unlock()

	f.clients = append(f.clients, c)

	dataMap := map[string]interface{}{}
	json.Unmarshal(f.RawData, &dataMap)
	return c.WriteJSON(msg.SendMessage{OP: "document", Data: dataMap})

}

// ClientRemove remove client from Session
func (f *FlowSession) ClientRemove(c *websocket.Conn) {
	f.Lock()
	defer f.Unlock()

	for i, cl := range f.clients {
		if cl == c {
			f.clients = append(f.clients[:i], f.clients[i+1:]...)
			break
		}
	}
}

// DocumentUpdate client c Updates the session document
func (f *FlowSession) DocumentUpdate(c *websocket.Conn, data []byte) error {
	f.Lock()
	defer f.Unlock()

	f.RawData = data // Update
	dataMap := map[string]interface{}{}
	json.Unmarshal(f.RawData, &dataMap)

	// Send to all except ours
	for _, sc := range f.clients {
		if sc == c { // ours
			continue
		}
		err := sc.WriteJSON(msg.SendMessage{OP: "document", Data: dataMap})
		if err != nil {
			return err
		}
	}
	return nil
}

// Document send document to client c
func (f *FlowSession) Document(c *websocket.Conn) error {
	f.Lock()
	defer f.Unlock()

	dataMap := map[string]interface{}{}
	json.Unmarshal(f.RawData, &dataMap)
	return c.WriteJSON(msg.SendMessage{OP: "document", Data: dataMap})
}
